import { type IterationResult } from '../useAsyncIter/index.js';
export { useAsyncIterMulti, type IterationResult, type IterationResultSet };
/**
 * `useAsyncIterMulti` hooks up multiple async iterables to your component and its lifecycle, letting
 * additional async iterables be added or removed on the go.
 *
 * Similar to `useAsyncIter`, only it works with zero or more async iterables and plain values instead of
 * a single one.
 *
 * @example
 * ```tsx
 * import { useAsyncIterMulti } from 'react-async-iterators';
 *
 * const wordGen = (async function* () {
 *   const words = ['Hello', 'React', 'Async', 'Iterators'];
 *   for (const word of words) {
 *     await new Promise(resolve => setTimeout(resolve, 1500));
 *     yield word;
 *   }
 * })();
 *
 * const fruitGen = (async function* () {
 *   const sets = [
 *     ['Peach', 'Mango', 'Orange'],
 *     ['Apple', 'Pear', 'Guyava'],
 *     ['Watermelon', 'Kiwi', 'Grapes'],
 *   ];
 *   for (const fruits of sets) {
 *     await new Promise(resolve => setTimeout(resolve, 2000));
 *     yield fruits;
 *   }
 * })();
 *
 * function MyComponent() {
 *   const [currentWords, currentFruits] = useAsyncIterMulti([wordGen, fruitGen]);
 *
 *   return (
 *     <div>
 *       <h2>
 *         Current word:
 *         {currentWords.pendingFirst
 *           ? 'Loading fruits...'
 *           : currentWords.error
 *           ? `Error: ${currentWords.error}`
 *           : currentWords.done
 *           ? `Done (last value: ${currentWords.value})`
 *           : `Value: ${currentWords.value}`}
 *       </h2>
 *       <ul>
 *         {currentFruits.pendingFirst
 *           ? 'Loading words...'
 *           : currentFruits.value.map(fruit => (
 *             <div key={fruit.name}>{fruit.name}</div>
 *           ))}
 *       </ul>
 *     </div>
 *   );
 * }
 * ```
 *
 * Given an array of async iterables for `inputs`, this hook will iterate over all of them concurrently,
 * updating (re-rendering) the host component whenever any yields a value, completes, or errors outs -
 * each time returning a combined array of all their current individual states, corresponding to their
 * original positions in the given `inputs`.
 * `inputs` may also be mixed with plain (non async iterable) values, in which case they'll simply be
 * returned as-are, coinciding along current values of other async iterables.
 * This can enable components that could seamlessly handle _"static"_ as well as _"changing"_ values
 * and props.
 *
 * The hook initializes and maintains its iteration process with each async iterable object as long as
 * that same object remains present in `inputs` arrays across subsequent updates. Changing the position
 * of such object in the array on a consequent call will __not__ close its current running iteration - it
 * will only change the position its result would appear at in the returned array.
 * Care should be taken therefore to not unintentionally recreate the given iterables on every render,
 * by e.g; declaring an iterable outside the component body, control __when__ it should be recreated
 * with React's [`useMemo`](https://react.dev/reference/react/useMemo) or alternatively use the library's
 * {@link iterateFormatted `iterateFormatted`} util for a formatted version of an iterable which
 * preserves its identity.
 * Whenever `useAsyncIterMulti` detects that one or more previously present async iterables have
 * disappeared from the `inputs` array, it will close their iteration processes.
 * On component unmount, the hook will also ensure closing all actively iterated async iterables.
 *
 * The array returned from `useAsyncIterMulti` contains all the individual most recent states of all
 * actively iterated objects and/or plain values from the current `inputs` (including each's most recent
 * value, who's completed, etc. - see {@link IterationResultSet `IterationResultSet`}).
 *
 * @template TValues The type of the set of all plain or async iterable input values an array.
 * @template TInitValues The type of all initial values corresponding to types of `TValues`.
 *
 * @param inputs An array of zero or more async iterable or plain values (mixed).
 * @param {object} opts An _optional_ object with options.
 * @param opts.initialValues An _optional_ array of initial values, in which each item provides a starting value for an async iterable on the same index in the `inputs` array. For every async iterable that has no corresponding initial value in this array, the default initial value is `undefined`.
 *
 * @returns An array of objects that provide up-to-date information about each input's current value, completion status, whether it's still waiting for its first value and so on, correspondingly with the order in which they appear on `inputs (see {@link IterationResultSet `IterationResultSet`}).
 *
 * @see {@link IterationResultSet `IterationResultSet`}
 *
 * @example
 * ```tsx
 * // Using `useAsyncIterMulti` with a dynamically-changing amount of inputs:
 *
 * import { useState } from 'react';
 * import { useAsyncIterMulti, type MaybeAsyncIterable } from 'react-async-iterators';
 *
 * function DynamicInputsComponent() {
 *   const [inputs, setInputs] = useState<MaybeAsyncIterable<string>[]>([]);
 *
 *   const states = useAsyncIterMulti(inputs);
 *
 *   const addAsyncIterValue = () => {
 *     const iterableValue = (async function* () {
 *       for (let i = 0; i < 10; i++) {
 *         await new Promise(resolve => setTimeout(resolve, 500));
 *         yield `Item ${i}`;
 *       }
 *     })();
 *     setInputs(prev => [...prev, iterableValue]);
 *   };
 *
 *   const addStaticValue = () => {
 *     const staticValue = `Static ${inputs.length + 1}`;
 *     setInputs(prev => [...prev, staticValue]);
 *   };
 *
 *   return (
 *     <div>
 *       <h3>Dynamic Concurrent Async Iteration</h3>
 *
 *       <button onClick={addAsyncIterValue}>🔄 Add Async Iterable</button>
 *       <button onClick={addStaticValue}>🗿 Add Static Value</button>
 *
 *       <ul>
 *         {states.map((state, i) => (
 *           <li key={i}>
 *             {state.done
 *               ? state.error
 *                 ? `Error: ${state.error}`
 *                 : 'Done'
 *               : state.pendingFirst
 *                 ? 'Pending...'
 *                 : `Value: ${state.value}`}
 *           </li>
 *         ))}
 *       </ul>
 *     </div>
 *   );
 * }
 * ```
 */
declare function useAsyncIterMulti<const TValues extends readonly unknown[], const TInitValues extends readonly unknown[] = readonly undefined[]>(inputs: TValues, opts?: {
    initialValues?: TInitValues;
}): IterationResultSet<TValues, TInitValues>;
type IterationResultSet<TValues extends readonly unknown[], TInitValues extends readonly unknown[] = undefined[]> = {
    [I in keyof TValues]: IterationResult<TValues[I], I extends keyof TInitValues ? TInitValues[I] : undefined>;
};
